/////////////////////////////////////////////////////////////////////
// Class Creator Version 2.0.000 Copyrigth (C) Poul A. Costinsky 1994
///////////////////////////////////////////////////////////////////
// Implementation File w32file.cpp
// class CWin32File
//
// 28/05/1995 19:41                             Author: Poul
///////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "critsect.h"
#include "findfile.h"

#include "w32file.h"


#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// class CWin32File

#define new DEBUG_NEW
//*****************************************************************
void CWin32File::ThrowOsError()
{
	CFileException::ThrowOsError((LONG)::GetLastError());
}
//*****************************************************************
// Default Constructor
//*****************************************************************
// Destructor
CWin32File::~CWin32File()
{
	Close();
}
//*****************************************************************
void	CWin32File::Open (LPCTSTR  lpszName,	// address of name of the file 
		    	DWORD  fdwAccess,	// access (read-write) mode 
		    	DWORD  fdwCreate,	// how to create 
		    	DWORD  fdwAttrsAndFlags,	// file attributes 
		    	DWORD  fdwShareMode,	// share mode 
		    	LPSECURITY_ATTRIBUTES  lpsa,	// address of security descriptor 
		    	HANDLE  hTemplateFile)	// handle of file with attributes to copy  
   
{
	m_H = 
	CreateFile (lpszName,	// address of name of the file 
		    fdwAccess,	// access (read-write) mode 
		    fdwShareMode,	// share mode 
		    lpsa,	// address of security descriptor 
		    fdwCreate,	// how to create 
		    fdwAttrsAndFlags,	// file attributes 
		    hTemplateFile);// handle of file with attributes to copy  
	
	if (m_H == INVALID_HANDLE_VALUE)
		ThrowOsError();
}
//*****************************************************************
void 	CWin32File::Close()
{
	if (m_H != INVALID_HANDLE_VALUE)
		VERIFY(::CloseHandle (m_H));
	m_H = INVALID_HANDLE_VALUE;
}
//*****************************************************************
void	CWin32File::Flush()
{
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	VERIFY(::FlushFileBuffers(m_H));
}
//*****************************************************************
void	CWin32File::SetEndOfFile()
{
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	if (!::SetEndOfFile(m_H));
		ThrowOsError();
}
//*****************************************************************
void	CWin32File::Seek (LONG lDistanceToMove, DWORD nFrom)
{
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	DWORD dwNew = ::SetFilePointer(m_H, lDistanceToMove, NULL, nFrom);
	if (dwNew  == (DWORD)-1)
		ThrowOsError();
}
//*****************************************************************
void	CWin32File::SetSize (LONG dwFileSizeLow, LONG dwFileSizeHigh)
{
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	DWORD dwNew = ::SetFilePointer(m_H, dwFileSizeLow, 
			&dwFileSizeHigh, FILE_BEGIN);
	if (dwNew  == (DWORD)-1)
		ThrowOsError();
	if(!::SetEndOfFile(m_H));
		ThrowOsError();
}
//*****************************************************************
DWORD	CWin32File::Read (LPVOID  lpBuffer, DWORD  nNumberOfBytesToRead)
{
	DWORD nRead = 0;
	
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	if (!::ReadFile ( m_H,	// handle of file to read 
		    lpBuffer,	// address of buffer that receives data  
		    nNumberOfBytesToRead,// number of bytes to read 
		    &nRead,		// address of number of bytes read 
		    NULL)		// address of structure for data 
	    )
		ThrowOsError();

   	return nRead;
}
//*****************************************************************
DWORD	CWin32File::Write(LPCVOID lpBuffer, DWORD  nNumberOfBytesToWrite)
{
	DWORD 	nWritten = 0,
		nWrittenTotal = 0;
	
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	const int nWriteStep = 65536 - 16;
	// F..d MicroSoft!!!!:
	// Write operations across a network are limited to 65535 bytes.
	const TCHAR* pBuf = (const TCHAR *)lpBuffer;
	do
		{
		DWORD toWrite = min (nWriteStep, nNumberOfBytesToWrite);
		if (!::WriteFile(m_H,		// handle of file to write to 
			    pBuf,	// address of data to write to file 
			    toWrite,	// number of bytes to write 
			    &nWritten,	// address of number of bytes written 
			    NULL)	// addr. of structure needed for overlapped I/O  
	   	    )
			ThrowOsError();
		ASSERT(nWritten == toWrite);
		nWrittenTotal += nWritten;
		pBuf 	      += nWritten;
		nNumberOfBytesToWrite -= nWritten;
		} while (nNumberOfBytesToWrite > 0);
	return nWrittenTotal;
}
//*****************************************************************
void CWin32File::GetFileInfo (LPBY_HANDLE_FILE_INFORMATION  lpFileInformation) const
{
	ASSERT(m_H != INVALID_HANDLE_VALUE);
	if (!::GetFileInformationByHandle(m_H, lpFileInformation))
		ThrowOsError();
}
//*****************************************************************
DWORD 	CWin32File::GetSize (LPDWORD  lpdwFileSizeHigh) const
{
	DWORD nSize = ::GetFileSize(m_H, lpdwFileSizeHigh);
	if (nSize == 0xFFFFFFFF)
		ThrowOsError();
	return nSize;
}
//*****************************************************************
///////////////////////////////////////////////////////////////////
// Static Functions
//*****************************************************************
void CWin32File::CopyFile (LPCTSTR  lpszExistingFile,
    			 	  LPCTSTR  lpszNewFile,
    			 	  BOOL  fFailIfExists)
{
	if (!::CopyFile (lpszExistingFile,lpszNewFile,fFailIfExists))
		ThrowOsError();
}
//*****************************************************************
void CWin32File::CreateDir (LPCTSTR lpszPath, LPSECURITY_ATTRIBUTES lpsa)
{
	if (!::CreateDirectory(lpszPath, lpsa))
		ThrowOsError();
}
//*****************************************************************
void CWin32File::DeleteFile (LPCTSTR  lpszExistingFile)
{
	if (!::DeleteFile (lpszExistingFile))
		ThrowOsError();
}
//*****************************************************************
CString CWin32File::CurrentDir()
{
	TCHAR 	CurDir [MaxPath];
	if (::GetCurrentDirectory (MaxPath,CurDir) == 0)
		ThrowOsError();
	return CString (CurDir);
}
//*****************************************************************
DWORD CWin32File::GetFileAttributes	(LPCTSTR  lpFileName)
{
	DWORD att = ::GetFileAttributes	(lpFileName);
	if (att == 0xFFFFFFFF)
		ThrowOsError();
	return att;
}
//*****************************************************************
CString CWin32File::GetTempPath()
{
	TCHAR 	Dir [MaxPath];
	if (::GetTempPath(MaxPath, Dir) == 0)
		ThrowOsError();
	return CString (Dir);
}
//*****************************************************************
CString CWin32File::GetTempFileName(LPCTSTR  lpszPath, LPCTSTR  lpszPrefix,
    					UINT  uUnique)
{
	TCHAR TempFile [MaxPath];
	static TCHAR CurrPath[] 	= ".";
	static TCHAR WizPrefix[] = "WIZ";
	if (lpszPath == NULL)
		lpszPath = CurrPath;
	if (lpszPrefix == NULL)
		lpszPrefix = WizPrefix;
	UINT ret = ::GetTempFileName(
			    lpszPath,	// address of name of directory where temp. file is created  
			    lpszPrefix,	// address of prefix of temp. filename 
			    uUnique,	// number used to create temp. filename 
			    TempFile 	// address of buffer that will receive temp. filename 
			   );
	if (ret == 0)
		ThrowOsError();
	
	return CString (TempFile);
}
//*****************************************************************
void CWin32File::MoveFile(LPCTSTR  lpszExisting, LPCTSTR  lpszNew, DWORD  fdwFlags)
{
	if (!::MoveFileEx (lpszExisting, lpszNew, fdwFlags))
		ThrowOsError();
}
//*****************************************************************
void CWin32File::DeleteFileAfterReboot (LPCTSTR  lpszExistingFile)
{
	CWin32File::MoveFile(lpszExistingFile, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
}
//*****************************************************************
void CWin32File::ChDir (LPCTSTR  lpszDir)
{
	if (!::SetCurrentDirectory(lpszDir))
		ThrowOsError();
}
//*****************************************************************
// Recursive Function to remove all files from current directory 
void CWin32File::RemoveAllFiles()
{
	static const TCHAR AllFiles[] 	= "*";
	static const TCHAR DirUp[] 	= "..";
	CWizFindFile Files (AllFiles);
	while (Files.Next())
		{
		if (Files.IsDir())
			{
			ChDir (Files.Name());
			RemoveAllFiles();
			ChDir (DirUp);
			RmDir (Files.Name(), FALSE);
			}
		else
			DeleteFile (Files.Name());
		} // while
}
//*****************************************************************
void CWin32File::RmDir (LPCTSTR  lpszDir, BOOL bSub)
{
	static CWizCriticalSection RmDirCriticalSection (FALSE);
	if (bSub)
		{
		CWizCritSect CS (RmDirCriticalSection);
		
		CString CurDir = CurrentDir();
		ChDir (lpszDir);
		RemoveAllFiles();
		ChDir (CurDir);
		}
	
	if (!::RemoveDirectory(lpszDir))
		ThrowOsError();
}
//*****************************************************************
void CWin32File::SetAttr (LPCTSTR  lpFileName,DWORD  dwFileAttributes)
{
	if (!::SetFileAttributes(lpFileName, dwFileAttributes))
		ThrowOsError();
}

//*****************************************************************
BOOL CWin32File::FileExist (LPCTSTR lpFileName, DWORD flags)
{
	DWORD att = ::GetFileAttributes	(lpFileName);
	if (att == 0xFFFFFFFF)
		return FALSE;
	if (att == FILE_ATTRIBUTE_NORMAL)
		return TRUE;
	
	if ((att & flags & ANY_FILE) != (att & ANY_FILE))
		return FALSE;
	return ((att & (flags & ~ANY_FILE)) == (flags & ~ANY_FILE));
}
//*****************************************************************
/////////////////////////////////////////////////////////////////////////////
//class CWizAsyncFileIO
//*****************************************************************
// Constructor starts Async. READING
CWizAsyncFileIO::CWizAsyncFileIO (CWin32File* pFile, LPVOID  lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh)
	: m_pFile (pFile)
{
	Init(nOffset, nOffsetHigh);
	Process (::ReadFile(
  			HANDLE(m_pFile),// handle of file to read 
		    	lpBuffer,	// address of buffer that receives data  
		    	nNumberOfBytesToRead,	// number of bytes to read 
		    	NULL,	// address of number of bytes read 
		    	&m_cOv 	// address of structure for data 
		   ));	
}
//*****************************************************************
// Constructor starts Async. WRITING---|
CWizAsyncFileIO::CWizAsyncFileIO (CWin32File* pFile,  LPCVOID  lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh)
	: m_pFile (pFile)
{
	Init(nOffset, nOffsetHigh);
	Process (::WriteFile(
  			HANDLE(m_pFile),// handle of file to read 
		    	lpBuffer,	// address of buffer that receives data  
		    	nNumberOfBytesToRead,	// number of bytes to read 
		    	NULL,	// address of number of bytes read 
		    	&m_cOv 	// address of structure for data 
		   ));	
}
//*****************************************************************
// Destructor
CWizAsyncFileIO::~CWizAsyncFileIO ()
{
	if (m_cOv.hEvent != NULL)
		{
		ASSERT(m_pFile->Valid());
		ASSERT(Test());
		::CloseHandle (m_cOv.hEvent);
		}
}
//*****************************************************************
void CWizAsyncFileIO::Init(DWORD nOffset, DWORD nOffsetHigh)
{
	ASSERT(m_pFile->Valid());
	m_bEndOfFileFound = FALSE;
	memset (&m_cOv, 0, sizeof(m_cOv));
	m_cOv.hEvent = ::CreateEvent (NULL,	// address of security attributes  
    				TRUE,	// flag for manual-reset event 
    				FALSE,	// flag for initial state 
    				NULL 	// address of event-object name  
   				);	
	if (m_cOv.hEvent == NULL)
		CWin32File::ThrowOsError();
	m_cOv.Offset 	 = nOffset;
	m_cOv.OffsetHigh = nOffsetHigh;
}
//*****************************************************************
// After starting Reading or Writing we must make some testing
void CWizAsyncFileIO::Process (BOOL bIOResult)
{
	// If I/O completed without pending, set event explicity to
	// signaled state
	if (bIOResult)
		::SetEvent (m_cOv.hEvent);
	else
		switch (::GetLastError())
			{
			// End of file found without pending
			case ERROR_HANDLE_EOF:
				m_bEndOfFileFound = TRUE;
				::SetEvent (m_cOv.hEvent);
				break;
			// Operation is pending, we'll wait
			case ERROR_IO_PENDING:
				break;
			// Other fault:
			default:
				CWin32File::ThrowOsError();
				break;
			} //switch
}
//*****************************************************************
DWORD CWizAsyncFileIO::Wait ()
{
	if (m_bEndOfFileFound)
		return 0;
	DWORD	dwBytes = 0;
	if (!::GetOverlappedResult(HANDLE(m_pFile),&m_cOv, &dwBytes, TRUE))
		switch (GetLastError())
			{
			case ERROR_HANDLE_EOF:
				return 0;
			default:
				CWin32File::ThrowOsError();
				return 0;
			} // switch
	return dwBytes;
}
//*****************************************************************
BOOL CWizAsyncFileIO::Test()
{
	if (m_bEndOfFileFound)
		return TRUE;
	
	DWORD	dwBytes = 0;
	if (!::GetOverlappedResult(HANDLE(m_pFile),&m_cOv, &dwBytes, FALSE))
		switch (GetLastError())
			{
			case ERROR_HANDLE_EOF:
				return TRUE;
			case ERROR_IO_INCOMPLETE:
				return FALSE;
			default:
				CWin32File::ThrowOsError();
				return FALSE;
			} // switch
	return TRUE;
}
//*****************************************************************


