/////////////////////////////////////////////////////////////////////
// Class Creator Version 2.0.000 Copyrigth (C) Poul A. Costinsky 1994
///////////////////////////////////////////////////////////////////
// Header File w32file.h
// class CWin32File
//
// 28/05/1995 19:41                             Author: Poul
///////////////////////////////////////////////////////////////////

#ifndef __CWin32File_H
#define __CWin32File_H

/////////////////////////////////////////////////////////////////////////////
//class CWizAsyncFileIO - class for asynchronous I/O (read/write)
// with CWin32File 
class CWizAsyncFileIO
{
friend class CWin32File;
private:
// Constructors only for CWin32File
	// Constructor starts Async. READING
	CWizAsyncFileIO (CWin32File* pFile, LPVOID  lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh);
	// Constructor starts Async. WRITING---|
	CWizAsyncFileIO (CWin32File* pFile,  LPCVOID  lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh);
public:
// Destructor
	~CWizAsyncFileIO();
// Operations
	// Function Waits until I/O operation completes
	// and returns number of bytes proceeded
	DWORD 	Wait ();
	// Finction tests completion of I/O operation
	BOOL	Test ();
private:
// Implementation
	void Init(DWORD nOffset, DWORD nOffsetHigh);
	void Process (BOOL bIOResult);
// Members
	CWin32File*	m_pFile;
	OVERLAPPED	m_cOv;
	BOOL		m_bEndOfFileFound;
	
};
/////////////////////////////////////////////////////////////////////////////
// class CWin32File
class CWin32File
{
public:
	enum	{ MaxPath = 1000, };
// Constructors:
	// Constructor with no file opened
	CWin32File () { m_H = INVALID_HANDLE_VALUE; }
	// Constructor with file opened already
	CWin32File (HANDLE h) { m_H = h; }
	// Constructor openes the file
	// For parameters see Win32 help for CreateFile function
	CWin32File (LPCTSTR  lpszName,	// name of the file 
	      DWORD  fdwAccess,	// access (read-write) mode 
	      DWORD  fdwCreate,	// how to create 
	      DWORD  fdwAttrsAndFlags,// file attributes 
	      DWORD  fdwShareMode = 0,// share mode; default - no sharing
	      LPSECURITY_ATTRIBUTES lpsa = NULL,// address of security descriptor 
	      HANDLE  hTemplateFile = NULL) 	// handle of file with attributes to copy  
		{
		Open   (lpszName, fdwAccess, fdwCreate,
      			fdwAttrsAndFlags, fdwShareMode,
	      		lpsa, hTemplateFile);
		}
// Destructor:
	virtual ~CWin32File ();
public:
	// returns handle of the opened file
	operator HANDLE() const { return m_H;}
	// test for valid state (file is opened succesfully
	BOOL operator !() const { return !Valid(); }
	BOOL Valid() const { return (m_H != INVALID_HANDLE_VALUE);}
// Operations:
	//************************************************************
	// Function opens the file. When fails, 
	// CFileException is thrown.
	// For parameters see Win32 help for CreateFile function
	void Open (LPCTSTR  lpszName,	// name of the file 
	      DWORD  fdwAccess,	// access (read-write) mode 
	      DWORD  fdwCreate,	// how to create 
	      DWORD  fdwAttrsAndFlags,// file attributes 
	      DWORD  fdwShareMode = 0,// share mode; default - no sharing
	      LPSECURITY_ATTRIBUTES lpsa = NULL,// address of security descriptor 
	      HANDLE  hTemplateFile = NULL);	// handle of file with attributes to copy  
	// Close the file
	void 	Close();
	// Flushes buffers to the disk
	void	Flush();
	// Moves end of the file mark to the current file position
	void	SetEndOfFile();

	//************************************************************
	// Function receives the file information. When fails, 
	// CFileException is thrown.
	void	GetFileInfo (LPBY_HANDLE_FILE_INFORMATION  lpFileInformation) const;
	// Get/Set size of the file
	DWORD 	GetSize (LPDWORD  lpdwFileSizeHigh = NULL) const;
	void	SetSize (LONG dwFileSizeLow, LONG dwFileSizeHigh = 0); 
	//************************************************************
	// Function sets the file pointer. When fails, 
	// CFileException is thrown.
	void	Seek (LONG lDistanceToMove, DWORD nFrom = FILE_BEGIN);

	// Function reads the file. When fails, 
	// CFileException is thrown.
	DWORD	Read (LPVOID  lpBuffer, DWORD  nNumberOfBytesToRead);
	// Function writes the file. When fails, 
	// CFileException is thrown.
	DWORD	Write(LPCVOID lpBuffer, DWORD  nNumberOfBytesToWrite);
	
	//************************************************************
	// Asynchronous READ/WRITE
	// Attention !!! File Must be open with  FILE_FLAG_OVERLAPPED,
	// and NO Synchronous I/O must be performed
	CWizAsyncFileIO	Read (LPVOID  lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh = 0)
		{
		return CWizAsyncFileIO (this,lpBuffer, nNumberOfBytesToRead,
					nOffset, nOffsetHigh);
		}
	CWizAsyncFileIO	Write(LPCVOID lpBuffer, DWORD  nNumberOfBytesToRead,
				DWORD nOffset, DWORD nOffsetHigh = 0)
		{
		return CWizAsyncFileIO (this,lpBuffer, nNumberOfBytesToRead,
					nOffset, nOffsetHigh);
		}
	//************************************************************
	// Static Operations
	enum   { ANY_FILE = 
			FILE_ATTRIBUTE_HIDDEN |
			FILE_ATTRIBUTE_READONLY	|
			FILE_ATTRIBUTE_SYSTEM
		};		
	static BOOL FileExist (LPCTSTR lpFileName, DWORD flags = FILE_ATTRIBUTE_NORMAL);
	static void CopyFile
			(LPCTSTR  lpszExistingFile,	// address of name of an existing file 
    			 LPCTSTR  lpszNewFile,		// address of filename to copy to 
    			 BOOL  fFailIfExists = FALSE 	// flag for operation if file exists 
   			);	
	static void CreateDir (
 			LPCTSTR  lpszPath,		  // pointer to a directory path string
    			LPSECURITY_ATTRIBUTES lpsa = NULL // pointer to a security descriptor 
   			);
	static void DeleteFile
			(LPCTSTR  lpszExistingFile	// address of name of an existing file 
    			 );	
	static void DeleteFileAfterReboot
			(LPCTSTR  lpszExistingFile	// address of name of an existing file 
    			 );
	static CString CurrentDir();
	static DWORD GetFileAttributes	(
    			LPCTSTR  lpFileName // address of the name of a file or directory  
   					);
	static CString GetTempPath();
	static CString GetTempFileName(
			LPCTSTR  lpszPath   = NULL,// address of name of directory where temp. file is created  
    			LPCTSTR  lpszPrefix = NULL,// address of prefix of temp. filename 
    			UINT  uUnique = 0	   // number used to create temp. filename 
			);
	static void MoveFile(
    			LPCTSTR  lpszExisting,	// address of name of the existing file  
    			LPCTSTR  lpszNew, 	// address of new name for the file 
						// If this parameter is NULL, the 
						// file specified by lpszExisting is deleted for 
						// "delay until reboot" operations only.
    			DWORD  fdwFlags = 	// flag to determine how to move file 
   			MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING
   			);
	static void RemoveAllFiles();
	static void RmDir (LPCTSTR  lpszDir, 	// address of directory to remove  
			BOOL	bSub = FALSE	// Forces to remove all files and subdirectories
						// from directory
   			);
	static void ChDir (LPCTSTR  lpszDir);

	static void SetAttr (
			LPCTSTR  lpFileName,	// address of filename 
    			DWORD  dwFileAttributes 	// address of attributes to set 
   			);
	static void ThrowOsError();
// Virtual operations:
protected:
// Implementation:
// Virtual implementation:
// Members
	HANDLE	m_H;
};

/////////////////////////////////////////////////////////////////////////////
// template class - typed implementation of the CWin32File
// TYPE can be any type  - built-in like int, or struct - 
// which can be read/written binary - i.e. hasn't virtual
// functions or nonbinary parents/data members
template <class TYPE>
class CWizRecordFile : protected CWin32File
{
public:
	enum { CURRENT_POSITION = -1 };
// Constructors:
	CWizRecordFile() : m_nPos (CURRENT_POSITION)
		 {}
	
	CWizRecordFile (LPCTSTR  lpszName,	// name of the file 
	      DWORD  fdwAccess,			// access (read-write) mode 
	      DWORD  fdwCreate,			// how to create 
	      DWORD  fdwAttrsAndFlags,		// file attributes 
	      DWORD  fdwShareMode = 0,		// share mode; default - no sharing
	      LPSECURITY_ATTRIBUTES lpsa = NULL)// handle of file with attributes to copy  
	:  m_nPos (0),
	   CWin32File (lpszName, fdwAccess, fdwCreate,
      			fdwAttrsAndFlags, fdwShareMode,lpsa)
		{}

public:
// Operations:
	void Open (LPCTSTR  lpszName,		// name of the file 
	      DWORD  fdwAccess,			// access (read-write) mode 
	      DWORD  fdwCreate,			// how to create 
	      DWORD  fdwAttrsAndFlags,		// file attributes 
	      DWORD  fdwShareMode = 0,		// share mode; default - no sharing
	      LPSECURITY_ATTRIBUTES lpsa = NULL)// address of security descriptor 
		{
		CWin32File::Open (lpszName, fdwAccess, fdwCreate,
      			fdwAttrsAndFlags, fdwShareMode,lpsa);
		m_nPos = 0;
		}
	LONG	GetSize() const
		{return LONG(CWin32File::GetSize()/sizeof(TYPE));}
	void	SetPos (LONG nPos)
		{ 
		if (m_nPos == nPos) return;
		m_nPos = nPos;
		Seek (m_nPos*sizeof(TYPE));
		}
	LONG	Read (TYPE* pBuffer, LONG nCount = 1, LONG nPos = CURRENT_POSITION)
		{
		if (nPos != CURRENT_POSITION)
			SetPos (nPos);
		LONG nRec = LONG(CWin32File::Read (pBuffer,nCount*sizeof(TYPE)))/sizeof(TYPE);
		m_nPos += nRec;
		return nRec;
		}
	LONG	Write (const TYPE* pBuffer, LONG nCount = 1, LONG nPos = CURRENT_POSITION)
		{
		if (nPos != CURRENT_POSITION)
			SetPos (nPos);
		LONG nRec = LONG(CWin32File::Write (pBuffer,nCount*sizeof(TYPE)))/sizeof(TYPE);
		m_nPos += nRec;
		return nRec;
		}
protected:
// Implementation:
// Virtual implementation:
// Members
	LONG	m_nPos;
};
typedef CWin32File* PWin32File;
/////////////////////////////////////////////////////////////////////////////
// Safe pointer to CWin32File object.
// Use this class elsewhere instead of CWin32File* pFile = new CWin32File;
// The operations replicate CWin32File functionality
class CWin32FilePtr
{
public:
// Constructors:
	CWin32FilePtr () : m_pFile (NULL) {}
	CWin32FilePtr (HANDLE h) : m_pFile (new CWin32File (h)) 
		{ if (m_pFile == NULL) AfxThrowMemoryException(); }
	CWin32FilePtr (
		LPCTSTR  lpszName,	// name of the file 
	      	DWORD  fdwAccess,	// access (read-write) mode 
	      	DWORD  fdwCreate,	// how to create 
	      	DWORD  fdwAttrsAndFlags,// file attributes 
	      	DWORD  fdwShareMode = 0,// share mode; default - no sharing
	      	LPSECURITY_ATTRIBUTES lpsa = NULL,// address of security descriptor 
	      	HANDLE  hTemplateFile = NULL, 	// handle of file with attributes to copy  
		BOOL	bCreate = TRUE)
	  : m_pFile (NULL)
		{
		if (bCreate)
		   Open(lpszName, fdwAccess, fdwCreate,
      			fdwAttrsAndFlags, fdwShareMode,
	      		lpsa, hTemplateFile);
		}
// Destructor:
	~CWin32FilePtr() { delete m_pFile; }
// Operations:
	operator PWin32File() { return m_pFile; }
	void Open (LPCTSTR  lpszName,	// name of the file 
	      DWORD  fdwAccess,	// access (read-write) mode 
	      DWORD  fdwCreate,	// how to create 
	      DWORD  fdwAttrsAndFlags,// file attributes 
	      DWORD  fdwShareMode = 0,// share mode; default - no sharing
	      LPSECURITY_ATTRIBUTES lpsa = NULL,// address of security descriptor 
	      HANDLE  hTemplateFile = NULL)	// handle of file with attributes to copy  
		{
		if (m_pFile != NULL)
			delete m_pFile;
		m_pFile = new CWin32File (
			lpszName, fdwAccess, fdwCreate,
      			fdwAttrsAndFlags, fdwShareMode,
	      		lpsa, hTemplateFile);
		if (m_pFile == NULL) AfxThrowMemoryException();
		}
	void	Close()
		{
		delete m_pFile;
		m_pFile = NULL;
		}
private:
// Members:
	CWin32File*	m_pFile;
};
/////////////////////////////////////////////////////////////////////////////
#endif // __CWin32File_H


